export { divide, minus, plus, times } from 'number-precision';

type GetField<Target, Key extends keyof any, Default> = Key extends keyof Target ? Target[Key] : Key extends `${number}` ? "length" extends keyof Target ? number extends Target["length"] ? number extends keyof Target ? Target[number] : Default : Default : Default : Default;
type GetRestPath<Rest extends string, FieldKey extends string, IndexKey extends string> = FieldKey extends "" ? `${Rest}[${IndexKey}]` : `${Rest}[${FieldKey}][${IndexKey}]`;
type ToPath<Target extends string, Rest extends string = ""> = Target extends `${infer Left}.${infer Right}` ? Left extends `${infer FieldKey}[${infer IndexKey}]` ? ToPath<Right, GetRestPath<Rest, FieldKey, IndexKey>> : ToPath<Right, `${Rest}[${Left}]`> : Target extends `${infer FieldKey}[${infer IndexKey}]` ? GetRestPath<Rest, FieldKey, IndexKey> : Target extends `[${infer Target}]` ? `${Rest}${Target}` : `${Rest}[${Target}]`;
type GetFieldDeepBase<Target, Path, Default> = Path extends `[${infer FieldKey}][${infer Right}]` ? GetFieldDeepBase<GetField<Target, FieldKey, Default>, `[${Right}]`, Default> : Path extends `[${infer FieldKey}]` ? GetField<Target, FieldKey, Default> : Default;
type GetFieldDeep<Target, Path extends string, Default> = GetFieldDeepBase<Target, ToPath<Path>, Default>;
type MergeInsertions<T> = T extends object ? {
    [K in keyof T]: MergeInsertions<T[K]>;
} : T;
type DeepMerge<F, S> = MergeInsertions<{
    [K in keyof F | keyof S]: K extends keyof S & keyof F ? DeepMerge<F[K], S[K]> : K extends keyof S ? S[K] : K extends keyof F ? F[K] : never;
}>;

type Insertions<T> = T extends object ? {
    [K in keyof T]: T[K];
} : T;
type Tree<T extends Record<any, any>, Child extends string = "children"> = {
    [k in keyof T | GetConst<Child, "children">]?: k extends Child ? Insertions<Tree<T, Child>>[] : k extends keyof T ? T[k] : never;
};
type convertTreeOpt = {
    pid?: string;
    id?: string;
    children?: string;
    rootId?: number;
};
type GetConst<T, S> = T extends string ? T : S;

type ExcludeField<T, K> = {
    [P in Extract<keyof T, K>]: T[P];
};
interface MapInstance<K, V> extends ExcludeField<Map<K, V>, "clear" | "delete" | "get" | "has" | "entries" | "keys" | "values" | "set" | "size"> {
    [Symbol.toStringTag]?: string;
    [Symbol.iterator]?(): IterableIterator<any>;
    forEach(callbackfn: MapEachCb<K, V>, thisArg?: any): void;
}
type MapEachCb<K, V> = (value: V, key: K, map: MapInstance<K, V>) => void;
interface MapConstructor {
    new (): MapInstance<any, any>;
    new <K, V>(entries?: readonly (readonly [K, V])[] | null): MapInstance<K, V>;
    prototype: MapInstance<any, any>;
    readonly [Symbol.species]?: MapConstructor;
}

interface Memoizify<U extends (...args: any) => any> {
    (...args: Parameters<U>): ReturnType<U>;
    cache?: MapInstance<any, any>;
}

type URL2 = {
    href: string | undefined;
    origin: string | undefined;
    protocol: string | undefined;
    host: string | undefined;
    hostName: string | undefined;
    port: string | undefined;
    pathname: string | undefined;
    search: string | undefined;
    hash: string | undefined;
};

type CreateContextEval = () => {
    remove: () => void;
    run: (src: string, scopes: Record<any, any>, options?: {
        type?: "module" | "script" | "AsyncFunction";
    }) => Promise<unknown>;
};
declare class Compartment {
    scopes: {};
    runningEvalContext: ReturnType<CreateContextEval>;
    constructor(scopes?: {});
    destroy(): void;
    evaluate(jsCode: string, options?: {}): Promise<unknown>;
}

/**
 * 获取类型 Tag
 * @param value
 */
declare const getTypeTag: (value: any) => string;
/**
 * 获取类型
 * @param value
 */
declare const getType: (value: any) => string;
/**
 * 判断类型是否 NaN
 * @param value
 */
declare const isNaN: (value: any) => boolean;
/**
 * 判断类型是否 Null
 * @param value
 */
declare const isNull: (value: any) => value is null;
/**
 * 判断类型是否 Undefined
 * @param value
 */
declare const isUndefined: (value: any) => value is undefined;
/**
 * 判断类型是否 Boolean
 * @param value
 */
declare const isBoolean: (value: any) => value is boolean;
/**
 * 判断类型是否 BigInt
 * @param value
 */
declare const isBigInt: (value: any) => value is bigint;
/**
 * 判断类型是否 Number
 * @param value
 */
declare const isNumber: (value: any) => value is number;
/**
 * 判断类型是否 String
 * @param value
 */
declare const isString: (value: any) => value is string;
/**
 * 判断类型是否 Symbol
 * @param value
 */
declare const isSymbol: (value: any) => value is symbol;
/**
 * 判断是不是纯粹的对象
 * @param value
 */
declare const isPrimeObject: (value: any) => value is object;
/**
 * 判断是否 Object
 * @param value
 */
declare const isObject: (value: any) => boolean;
/**
 * 判断类型是否 RegExp
 * @param value
 */
declare const isRegExp: (value: any) => value is RegExp;
/**
 * 判断类型是否 Date
 * @param value
 */
declare const isDate: (value: any) => value is Date;
/**
 * 判断类型是否 Array
 * @param value
 */
declare const isArray: (value: any) => value is [];
/**
 * 判断类型是否 Function
 * @param value
 */
declare const isFunction: (value: any) => value is () => void;
/**
 * 判断类型是否 Promise
 * @param value
 */
declare const isPromise: (value: any) => value is Promise<any>;
/**
 * 判断是否整数
 * @param value
 */
declare const isInteger: (value: any) => boolean;
/**
 * 数值是否在有效的数组安全长度范围
 * 大于等于0并且小于MAX_SAFE_INTEGER
 * @param value
 */
declare const isArraySafeLength: (value: any) => boolean;
/**
 * 判断是否是类数组的对象
 * 1：不为null, undefined
 * 2：为object
 * 3: length属性为数组安全长度
 * @param value
 */
declare const isArrayLike: (value: any) => boolean;
/**
 * 判断元素是否为空
 * @param value
 */
declare const isEmpty: (value: any) => boolean;
/**
 * 判断对象自身属性中是否具有指定的属性
 * @param target
 * @param key 属性
 */
declare const hasOwn: <T extends Record<string, any>>(target: T, key: string) => boolean;
/**
 * 判断是否支持 iterator
 */
declare const hasIterable: boolean;

/**
 * 将数值转换为整数
 * @param {number} value
 * @returns {number}
 */
declare const toInteger: (value: number) => number;
/**
 * 将数值转为数组安全长度
 * @param {number} value
 * @returns {number}
 */
declare const toArraySafeLength: (value: number) => number;
/**
 * 转换为数组
 * @param {any} value
 * @returns
 */
declare const toArray: (value: any) => any[];
/**
 * 转换为 string 类型
 * @param {any} value
 * @returns {string}
 */
declare const toString: (value: any) => string;
/**
 * 转换为数字
 * @param {any} value
 * @returns {number}
 */
declare const toNumber: (value: any) => number;
/**
 * 将值转换为有限数字
 * @param {any} value
 * @returns {number}
 */
declare const toFinite: (value: number) => number;

/**
 * 指定范围(min <= value <= max)内的随机整数
 * @param min 最小值
 * @param max 最大值
 */
declare const randomInteger: (min: number, max: number) => number;
/**
 * 将数字四舍五入到指定的小数位数
 * @param value 数值
 * @param 精确到几位小数, 默认为 0
 */
declare const round: (value: number, decimal?: number) => number;

/**
 * 数据脱敏处理
 * @param value 目标字符串
 * @param startIndex 起始处的索引
 * @param endIndex 终止处的索引
 * @param symbol 替换的字符串，默认为*
 */
declare const sensitive: (value: string, startIndex: number, endIndex: number, symbol?: string) => string;
declare const mask: (value: string, startIndex: number, endIndex: number, symbol?: string) => string;
/**
 * 从位置左边开始对数据脱敏处理
 * @param value 目标字符串
 * @param index 索引（从左边开始）
 * @param symbol 替换的字符串，默认为*
 */
declare const sensitiveLeft: (value?: string, index?: number, symbol?: string) => string;
declare const maskLeft: (value?: string, index?: number, symbol?: string) => string;
/**
 * 从位置右边开始对数据脱敏处理
 * @param value 目标字符串
 * @param index 索引（从右边开始）
 * @param symbol 替换的字符串，默认为*
 */
declare const sensitiveRight: (value?: string, index?: number, symbol?: string) => string;
declare const maskRight: (value?: string, index?: number, symbol?: string) => string;
/**
 * 全局唯一标识符 UUID
 * @param len 长度
 * @param radix 基数(默认：62)
 */
declare const uuid: (len: number, radix: number) => string;
/**
 * 128位的数字标识符
 */
declare const guid: () => string;
/**
 * 将数值转化为千分位格式
 * @param value 数值
 * @param symbol 前缀符号
 */
declare const toThousands: (value: string | number, symbol?: string) => string;
/**
 * 去除开头结尾空格
 * @param {string} value
 * @returns
 */
declare const trim: (value: string) => string;

/**
 * 根据属性去重数组
 * @param array 目标数组
 * @param key 属性
 */
declare const uniqueBy: <T>(array: T[], key?: string) => T[];
/** 普通数组去重 */
declare const unique: (arr: any[]) => any[];
/**
 * 根据属性找出数组中最大值的一列
 * @param array 目标数组
 * @param key 属性
 */
declare const maxNumBy: <T extends Record<any, any>>(array: T[], key: string) => T | undefined;
/**
 * 根据属性找出数组中最小值的一列
 * @param array 目标数组
 * @param key 属性
 */
declare const minNumBy: <T extends Record<any, any>>(array: T[], key: string) => T | undefined;
/**
 * 将数组打乱
 * @param array 目标数组
 */
declare const shuffle: <T>(array: T[]) => T[];
/**
 * 扁平化数组 ==> 树形结构
 * @param list 数组List
 * @param option 配置
 * @param option.pid 父节点id属性名
 * @param option.id 节点id属性名
 * @param option.children 节点children属性名
 * @param option.rootId 根节点id
 */
declare const convertTree: <List extends Record<string, any>, Child extends string = "children">(list?: List[], option?: convertTreeOpt) => Tree<List, Child>[];
declare const convert: <List extends Record<string, any>, Child extends string = "children">(list?: List[], option?: convertTreeOpt) => Tree<List, Child>[];
/**
 * 树形结构 ==> 扁平化
 * @param nodes 节点树
 * @param childrenName 节点children属性名
 */
declare const treeFlat: <T extends Record<any, any>, Child extends string = "children">(nodes: Tree<T, Child>[], childrenName?: Child) => T[];
declare const convertFlat: <T extends Record<any, any>, Child extends string = "children">(nodes: Tree<T, Child>[], childrenName?: Child) => T[];
/**
 * 树形结构转路径集合
 * @param nodes 节点树
 * @param childrenName 节点children属性名
 * @param path 路径
 * @param paths 路径集合
 */
declare const convertFlatPaths: <T extends Record<any, any>, Child extends string = "children">(nodes: Tree<T, Child>[], childrenName?: Child, path?: T[], paths?: T[][]) => T[][];
/**
 * 根据 key 递归查找链带关系
 * @param leafKey 节点属性名
 * @param leafValue 节点属性值
 * @param nodes 节点树
 * @param childrenName 节点children属性名
 * @param path 链带路径
 */
declare const findPathByLeaf: <T extends Record<any, any>, Child extends string = "children">(leafKey: string, leafValue: any, nodes: Tree<T, Child>[], childrenName?: Child, path?: T[]) => any;
/**
 * 返回按属性(sortKey)和顺序(sortBy)排序的对象数组。
 * @param array 目标数组
 * @param sortKey 排序属性
 * @param sortBy 排序方式 'desc降序' 、 'asc升序'
 */
declare const orderBy: <T, sortKey extends keyof T>(array: T[], sortKey: sortKey[], sortBy: ("asc" | "desc")[]) => T[];
/**
 * while 遍历
 * @param array 目标数组
 * @param cb 遍历回调
 */
declare function whileEach<T>(array: T[], cb: (curr: T, index: number, array: T[]) => any): void;

/**
 * 函数缓存
 * @param func 缓存目标函数
 * @param resolver
 * @returns 目标函数结果
 */
declare function memoize<U extends Memoizify<U>>(this: any, func: U, resolver?: () => void): Memoizify<U>;
/**
 * queryVar 查找变量值
 * @param condition 查询函数
 * @param delay 查询轮询时间
 */
declare const queryVar: <Resolve>(condition: () => Resolve, delay?: number) => Promise<Resolve>;
/**
 * sleep 休眠
 * @param delay 休眠时间
 */
declare const sleep: (delay: number) => Promise<NodeJS.Timeout>;
/**
 * 函数防抖
 * @param fn 函数
 * @param delay 延迟执行毫秒数
 * @param immediate 是否立即调用，默认false
 */
declare const debounce: (fn: (...args: any[]) => void, delay: number, immediate?: boolean) => (this: any, ...args: any[]) => void;
/**
 * 函数防抖 (立即执行)
 * @param fn 函数
 * @param delay 延迟执行毫秒数
 */
declare const debounceStart: (fn: any, delay?: number) => (this: any, ...args: any[]) => void;
/**
 * 函数防抖 (非立即执行)
 * @param fn 函数
 * @param delay 延迟执行毫秒数
 */
declare const debounceEnd: (fn: any, delay?: number) => (this: any, ...args: any[]) => void;
/**
 * 函数节流
 * @param fn 函数
 * @param delay 延迟执行毫秒数
 */
declare const throttle: (fn: (...args: any[]) => any, delay: number) => (this: any, ...args: any[]) => void;

/**
 * 深度克隆
 * @param target 目标对象
 */
declare const deepClone: <T>(target: T, map?: Map<any, any>) => T | null;
/**
 * 对象合并，与Object.assign不同的是，不覆盖前边已有属性的值
 * @param target 目标对象
 * @param source 来源对象
 */
declare const merge: <T extends Record<any, any>, S extends Record<any, any>>(target: T, source: S) => DeepMerge<T, S>;
/**
 * 深度获取属性值
 * @param target 目标对象
 * @param path 路径
 * @param defaultValue 默认值
 */
declare const get: <TObject extends Record<any, any>, TPath extends string, TDefailt>(target: TObject, path: TPath, defaultValue?: TDefailt | undefined) => GetFieldDeep<TObject, TPath, TDefailt>;
declare const selector: <TObject extends Record<any, any>, TPath extends string, TDefailt>(target: TObject, path: TPath, defaultValue?: TDefailt | undefined) => GetFieldDeep<TObject, TPath, TDefailt>;

/**
 * file、blob 转 DateURL
 * @param stream
 */
declare const blobToDataURL: (stream: Blob | File) => Promise<any>;
/**
 * dataURL 转为 Blob 格式
 * @param {String} dataURL
 * @return {Blob}
 */
declare const dataURLtoBlob: (dataURL: string) => Blob;
/**
 * 将图片的 url 转为 base64 格式
 * @param {string} url
 * @returns {Promise<any>}
 */
declare const getImgToBase64: (url: string) => Promise<any>;
/**
 * 将图片的 url 转为 Blob 格式
 * @param {string} url
 * @returns {Promise<Blob>}
 */
declare const getImgToBlob: (url: string) => Promise<Blob>;
/**
 * 根据内容下载文件
 * @param {string} fileName 文件名
 * @param {string} content 文件内容
 * @param {string} type 文件类型
 */
declare const downloadContent: (fileName: string, content: string, type?: string) => void;
/**
 * 将文本数据放置在剪贴板上
 * @param {String} value
 */
declare const setClipboardData: (value: string) => void;
/**
 * 动态引入
 * @param path 资源地址
 * @param bundleId 资源的ID
 * @param cb 请求资源成功后的回调
 * @returns
 */
declare const dynamicImport: <T>(path: string, bundleId: string, cb?: () => any) => Promise<T>;

/**
 * url 参数字符串反序列化
 * @param url
 */
declare function urlSearchParams(url: string): Record<string, string>;
/**
 * url 参数序列化
 * @param value
 */
declare const urlSearchStringify: (params: Record<string, any>) => string;
/**
 * 返回网址的相关信息，模拟了 浏览器的 new URL(url)
 * @param {string} url 网址
 */
declare const url: (url: string) => URL2;

/**
 * 时间格式化
 * @param value 时间
 * @param format 格式
 */
declare const formatTime: (value: any, format: string) => string;
/**
 * 将时间戳转为 xx小时xx分钟xx秒 例如1h0m10s
 * @param value 时间戳
 */
declare const formatHMS: (value: number) => string;
/**
 * 获取时间戳 (秒)
 * @param value
 */
declare const unix: (value?: number) => number;
/**
 * 倒计时
 * @param value 时间戳
 */
declare const countDown: (value: number) => string;

declare const MapCtor: MapConstructor;

/**
 * base64 加密
 * @param {string} input
 */
declare const encode: (input?: string) => string;
/**
 * base64 解密
 * @param {string} input
 */
declare const decode: (input?: string) => string;

declare const _default: {
    encode: (input?: string) => string;
    decode: (input?: string) => string;
    formatTime: (value: any, format: string) => string;
    formatHMS: (value: number) => string;
    unix: (value?: number | undefined) => number;
    countDown: (value: number) => string;
    urlSearchParams(url: string): Record<string, string>;
    urlSearchStringify: (params: Record<string, any>) => string;
    url: (url: string) => URL2;
    memoize<U extends Memoizify<U>>(this: any, func: U, resolver?: (() => void) | undefined): Memoizify<U>;
    queryVar: <Resolve>(condition: () => Resolve, delay?: number) => Promise<Resolve>;
    sleep: (delay: number) => Promise<NodeJS.Timeout>;
    debounce: (fn: (...args: any[]) => void, delay: number, immediate?: boolean) => (this: any, ...args: any[]) => void;
    debounceStart: (fn: any, delay?: number) => (this: any, ...args: any[]) => void;
    debounceEnd: (fn: any, delay?: number) => (this: any, ...args: any[]) => void;
    throttle: (fn: (...args: any[]) => any, delay: number) => (this: any, ...args: any[]) => void;
    deepClone: <T>(target: T, map?: Map<any, any>) => T | null;
    merge: <T_1 extends Record<any, any>, S extends Record<any, any>>(target: T_1, source: S) => DeepMerge<T_1, S>;
    get: <TObject extends Record<any, any>, TPath extends string, TDefailt>(target: TObject, path: TPath, defaultValue?: TDefailt | undefined) => GetFieldDeep<TObject, TPath, TDefailt>;
    selector: <TObject extends Record<any, any>, TPath extends string, TDefailt>(target: TObject, path: TPath, defaultValue?: TDefailt | undefined) => GetFieldDeep<TObject, TPath, TDefailt>;
    toInteger: (value: number) => number;
    toArraySafeLength: (value: number) => number;
    toArray: (value: any) => any[];
    toString: (value: any) => string;
    toNumber: (value: any) => number;
    toFinite: (value: number) => number;
    getTypeTag: (value: any) => string;
    getType: (value: any) => string;
    isNaN: (value: any) => boolean;
    isNull: (value: any) => value is null;
    isUndefined: (value: any) => value is undefined;
    isBoolean: (value: any) => value is boolean;
    isBigInt: (value: any) => value is bigint;
    isNumber: (value: any) => value is number;
    isString: (value: any) => value is string;
    isSymbol: (value: any) => value is symbol;
    isPrimeObject: (value: any) => value is object;
    isObject: (value: any) => boolean;
    isRegExp: (value: any) => value is RegExp;
    isDate: (value: any) => value is Date;
    isArray: (value: any) => value is [];
    isFunction: (value: any) => value is () => void;
    isPromise: (value: any) => value is Promise<any>;
    isInteger: (value: any) => boolean;
    isArraySafeLength: (value: any) => boolean;
    isArrayLike: (value: any) => boolean;
    isEmpty: (value: any) => boolean;
    hasOwn: <T_2 extends Record<string, any>>(target: T_2, key: string) => boolean;
    hasIterable: boolean;
    blobToDataURL: (stream: Blob | File) => Promise<any>;
    dataURLtoBlob: (dataURL: string) => Blob;
    getImgToBase64: (url: string) => Promise<any>;
    getImgToBlob: (url: string) => Promise<Blob>;
    downloadContent: (fileName: string, content: string, type?: string) => void;
    setClipboardData: (value: string) => void;
    dynamicImport: <T_3>(path: string, bundleId: string, cb?: (() => any) | undefined) => Promise<T_3>;
    plus: (...nums: (string | number)[]) => number;
    minus: (...nums: (string | number)[]) => number;
    times: (...nums: (string | number)[]) => number;
    divide: (...nums: (string | number)[]) => number;
    randomInteger: (min: number, max: number) => number;
    round: (value: number, decimal?: number) => number;
    whileEach<T_4>(array: T_4[], cb: (curr: T_4, index: number, array: T_4[]) => any): void;
    uniqueBy: <T_5>(array: T_5[], key?: string | undefined) => T_5[];
    unique: (arr: any[]) => any[];
    maxNumBy: <T_6 extends Record<any, any>>(array: T_6[], key: string) => T_6 | undefined;
    minNumBy: <T_7 extends Record<any, any>>(array: T_7[], key: string) => T_7 | undefined;
    shuffle: <T_8>(array: T_8[]) => T_8[];
    convertTree: <List extends Record<string, any>, Child extends string = "children">(list?: List[], option?: convertTreeOpt | undefined) => Tree<List, Child>[];
    convert: <List extends Record<string, any>, Child extends string = "children">(list?: List[], option?: convertTreeOpt | undefined) => Tree<List, Child>[];
    treeFlat: <T_9 extends Record<any, any>, Child_1 extends string = "children">(nodes: Tree<T_9, Child_1>[], childrenName?: Child_1) => T_9[];
    convertFlat: <T_9 extends Record<any, any>, Child_1 extends string = "children">(nodes: Tree<T_9, Child_1>[], childrenName?: Child_1) => T_9[];
    convertFlatPaths: <T_10 extends Record<any, any>, Child_2 extends string = "children">(nodes: Tree<T_10, Child_2>[], childrenName?: Child_2, path?: T_10[], paths?: T_10[][]) => T_10[][];
    findPathByLeaf: <T_11 extends Record<any, any>, Child_3 extends string = "children">(leafKey: string, leafValue: any, nodes: Tree<T_11, Child_3>[], childrenName?: Child_3, path?: T_11[]) => any;
    orderBy: <T_12, sortKey extends keyof T_12>(array: T_12[], sortKey: sortKey[], sortBy: ("asc" | "desc")[]) => T_12[];
    sensitive: (value: string, startIndex: number, endIndex: number, symbol?: string) => string;
    mask: (value: string, startIndex: number, endIndex: number, symbol?: string) => string;
    sensitiveLeft: (value?: string, index?: number, symbol?: string) => string;
    maskLeft: (value?: string, index?: number, symbol?: string) => string;
    sensitiveRight: (value?: string, index?: number, symbol?: string) => string;
    maskRight: (value?: string, index?: number, symbol?: string) => string;
    uuid: (len: number, radix: number) => string;
    guid: () => string;
    toThousands: (value: string | number, symbol?: string) => string;
    trim: (value: string) => string;
    version: string;
    Compartment: typeof Compartment;
};

export { Compartment, MapCtor, blobToDataURL, convert, convertFlat, convertFlatPaths, convertTree, countDown, dataURLtoBlob, debounce, debounceEnd, debounceStart, decode, deepClone, _default as default, downloadContent, dynamicImport, encode, findPathByLeaf, formatHMS, formatTime, get, getImgToBase64, getImgToBlob, getType, getTypeTag, guid, hasIterable, hasOwn, isArray, isArrayLike, isArraySafeLength, isBigInt, isBoolean, isDate, isEmpty, isFunction, isInteger, isNaN, isNull, isNumber, isObject, isPrimeObject, isPromise, isRegExp, isString, isSymbol, isUndefined, mask, maskLeft, maskRight, maxNumBy, memoize, merge, minNumBy, orderBy, queryVar, randomInteger, round, selector, sensitive, sensitiveLeft, sensitiveRight, setClipboardData, shuffle, sleep, throttle, toArray, toArraySafeLength, toFinite, toInteger, toNumber, toString, toThousands, treeFlat, trim, unique, uniqueBy, unix, url, urlSearchParams, urlSearchStringify, uuid, whileEach };
